@@ -67,13 +67,24 @@ class Colr:
     grey   = 1.0, 1.0, 1.0, 0.4
     black  = 0.0, 0.0, 0.0, 1.0
     yellow = 1.0, 1.0, 0.0, 0.6
-    brown  = 0.15, 0.15, 0.15, 0.20
 
 
-class RotDat:
+# Transformation Data
+class TransDat:
     placeholder = True
 
 
+def set_transform_data_none():
+    TransDat.piv_norm = None  # Vector
+    TransDat.new_ang_r = None
+    TransDat.ang_diff_r = None  # float
+    TransDat.axis_lock = None  # 'X', 'Y', 'Z'
+    TransDat.lock_pts = None
+    TransDat.rot_pt_pos = None
+    TransDat.rot_pt_neg = None
+    TransDat.arc_pts = None
+
+
 # Refreshes mesh drawing in 3D view and updates mesh coordinate
 # data so ref_pts are drawn at correct locations.
 # Using editmode_toggle to do this seems hackish, but editmode_toggle seems
@@ -470,22 +481,13 @@ def init_ref_pts(self):
         ReferencePoint("anc", Colr.red),
         ReferencePoint("piv", Colr.yellow)
     ]
-    # todo : move this part of initialization elsewhere?
-    RotDat.piv_norm = None
-    RotDat.new_ang_r = None
-    RotDat.ang_diff_r = None
-    RotDat.axis_lock = None
-    RotDat.lock_pts = None
-    RotDat.rot_pt_pos = None
-    RotDat.rot_pt_neg = None
-    RotDat.arc_pts = None
 
 
 def set_piv(self):
     #if self.pt_cnt == 2:
     if self.pt_cnt == 3:
-        rpts = [p.co3d for p in self.pts]
-        RotDat.piv_norm = geometry.normal(*rpts)
+        rpts = tuple(p.co3d for p in self.pts)
+        TransDat.piv_norm = geometry.normal(rpts)
 
 def set_mouse_highlight(self):
     if self.pt_cnt < 3:
@@ -541,7 +543,7 @@ def rem_ref_pt(self, idx):
     if self.pt_cnt > 1:
         updatelock_pts(self, self.pts)
     else:
-        RotDat.axis_lock = None
+        TransDat.axis_lock = None
     self.highlight_mouse = True
 
 
@@ -836,14 +838,14 @@ def set_arc_pts(ref_pts):
         temp.rotate(rot_val)
         arc_pts.append(temp + piv)
 
-    elif RotDat.axis_lock is not None:
-        #if RotDat.axis_lock == 'X':
+    elif TransDat.axis_lock is not None:
+        #if TransDat.axis_lock == 'X':
         #    rot_val = Euler((pi*2, 0.0, 0.0), 'XYZ')
-        if RotDat.axis_lock == 'X':
+        if TransDat.axis_lock == 'X':
             piv_norm = 1.0, 0.0, 0.0
-        elif RotDat.axis_lock == 'Y':
+        elif TransDat.axis_lock == 'Y':
             piv_norm = 0.0, 1.0, 0.0
-        elif RotDat.axis_lock == 'Z':
+        elif TransDat.axis_lock == 'Z':
             piv_norm = 0.0, 0.0, 1.0
         dis_p_f = (piv - fre).length
         dis_p_a = (piv - anc).length
@@ -862,7 +864,7 @@ def set_arc_pts(ref_pts):
             temp.rotate(rot_val)
             arc_pts.append(temp + piv)
 
-    RotDat.arc_pts = arc_pts
+    TransDat.arc_pts = arc_pts
 
 
 # Takes a ref_pts (ReferencePoints class) argument and modifies its member
@@ -873,9 +875,9 @@ def set_arc_pts(ref_pts):
 # todo : move inside ReferencePoints class ?
 def set_lock_pts(ref_pts, pt_cnt):
     if pt_cnt < 2:
-        RotDat.lock_pts = []
-    elif RotDat.axis_lock is None:
-        RotDat.lock_pts = ref_pts
+        TransDat.lock_pts = []
+    elif TransDat.axis_lock is None:
+        TransDat.lock_pts = ref_pts
         if pt_cnt == 3:
             set_arc_pts(ref_pts)
 
@@ -925,30 +927,30 @@ def create_z_orient(rot_vec):
 def updatelock_pts(self, ref_pts):
     global curr_meas_stor
     set_lock_pts(ref_pts, self.pt_cnt)
-    if RotDat.lock_pts == []:
-        if RotDat.axis_lock is not None:
-            self.report({'ERROR'}, 'Axis lock \''+ RotDat.axis_lock+
+    if TransDat.lock_pts == []:
+        if TransDat.axis_lock is not None:
+            self.report({'ERROR'}, 'Axis lock \''+ TransDat.axis_lock+
                     '\' creates identical points')
-        RotDat.lock_pts = ref_pts
-        RotDat.axis_lock = None
+        TransDat.lock_pts = ref_pts
+        TransDat.axis_lock = None
 
 
 # See if key was pressed that would require updating the axis lock info.
 # If one was, update the lock points to use new info.
 def axis_key_check(self, new_axis):
     if self.pt_cnt == 1:
-        if new_axis != RotDat.axis_lock:
-            RotDat.axis_lock = new_axis
+        if new_axis != TransDat.axis_lock:
+            TransDat.axis_lock = new_axis
 
 
 def draw_rot_arc(colr):
     reg = bpy.context.region
     rv3d = bpy.context.region_data
-    len_arc_pts = len(RotDat.arc_pts)
+    len_arc_pts = len(TransDat.arc_pts)
     if len_arc_pts > 1:
-        last = loc3d_to_reg2d(reg, rv3d, RotDat.arc_pts[0])
+        last = loc3d_to_reg2d(reg, rv3d, TransDat.arc_pts[0])
         for p in range(1, len_arc_pts):
-            p2d = loc3d_to_reg2d(reg, rv3d, RotDat.arc_pts[p])
+            p2d = loc3d_to_reg2d(reg, rv3d, TransDat.arc_pts[p])
             draw_line_2d(last, p2d, Colr.white)
             last = p2d
 
@@ -1031,18 +1033,18 @@ def draw_callback_px(self, context):
         rwid = context.region.width
         rhgt = context.region.height
         if self.pt_cnt == 1:
-            if RotDat.axis_lock is not None:
-                if self.running_transf is False:
+            if TransDat.axis_lock is not None:
+                if not self.running_transf:
                     self.rotate_btn.draw_btn(pts2d[0], self.mouse_co)
                     self.rotate_btn.is_drawn = True
 
-                if RotDat.axis_lock == 'X':
+                if TransDat.axis_lock == 'X':
                     test = self.pts[0].co3d + Vector((1, 0, 0))
                     colr = Colr.red
-                elif RotDat.axis_lock == 'Y':
+                elif TransDat.axis_lock == 'Y':
                     test = self.pts[0].co3d + Vector((0, 1, 0))
                     colr = Colr.green
-                elif RotDat.axis_lock == 'Z':
+                elif TransDat.axis_lock == 'Z':
                     test = self.pts[0].co3d + Vector((0, 0, 1))
                     colr = Colr.blue
 
@@ -1058,7 +1060,7 @@ def draw_callback_px(self, context):
                 blf.color(font_id, *colr)
                 blf.size(font_id, txt_sz, dpi)
                 blf.position(font_id, x_pos, y_pos, 0)
-                blf.draw(font_id, RotDat.axis_lock)
+                blf.draw(font_id, TransDat.axis_lock)
 
         elif self.pt_cnt == 2:
             axis_pts = get_axis_line_co(pts2d[0], pts2d[1], rwid, rhgt)
@@ -1073,7 +1075,7 @@ def draw_callback_px(self, context):
                     self.rotate_btn.draw_btn(btn_co, self.mouse_co)
                     self.rotate_btn.is_drawn = True
         elif self.pt_cnt == 3:
-            test = self.pts[2].co3d + RotDat.piv_norm
+            test = self.pts[2].co3d + TransDat.piv_norm
             t2d = loc3d_to_reg2d(reg, rv3d, test)
             axis_pts = get_axis_line_co(pts2d[2], t2d, rwid, rhgt)
             if axis_pts is not None:
@@ -1191,42 +1193,43 @@ class XEDIT_OT_free_rotate(bpy.types.Operator):
             #===========================
             # Safely exit transform
             #===========================
-            if self.running_transf is True:
+            if self.running_transf:
                 self.running_transf = False
 
             #===================================
             # Check for click on Rotate Button
             #===================================
-            elif self.rotate_btn.is_drawn is True and \
-                    self.rotate_btn.ms_over is True:
+            elif self.rotate_btn.is_drawn and self.rotate_btn.ms_over:
                 #print("Button Clicked")
                 curs_loc = None
+                rot_axis = None
                 #bpy.ops.object.ms_input_dialog_op('INVOKE_DEFAULT')
                 if self.pt_cnt == 1:
-                    if RotDat.axis_lock == 'X':
+                    if TransDat.axis_lock == 'X':
                         rot_axis = Vector((1.0, 0.0, 0.0))
-                    elif RotDat.axis_lock == 'Y':
+                    elif TransDat.axis_lock == 'Y':
                         rot_axis = Vector((0.0, 1.0, 0.0))
-                    elif RotDat.axis_lock == 'Z':
+                    elif TransDat.axis_lock == 'Z':
                         # -1 because it is assumed most rotations
                         # will have negative z pointing down
                         rot_axis = Vector((0.0, 0.0, -1.0))
                     curs_loc = self.pts[0].co3d.copy()
                 elif self.pt_cnt == 2:
-                    #if RotDat.axis_lock is None:
+                    #if TransDat.axis_lock is None:
                     rot_vec = self.pts[1].co3d - self.pts[0].co3d
                     rot_axis = rot_vec.normalized()
                     curs_loc = self.pts[0].co3d.lerp(self.pts[1].co3d, 0.5)
                 elif self.pt_cnt == 3:
-                    #if RotDat.axis_lock is None:
-                    rot_axis = RotDat.piv_norm
+                    #if TransDat.axis_lock is None:
+                    rot_axis = TransDat.piv_norm
                     curs_loc = self.pts[2].co3d.copy()
                 o_mat = create_z_orient(rot_axis)
                 self.running_transf = True
                 bpy.context.tool_settings.transform_pivot_point = 'CURSOR'
                 bpy.context.scene.cursor.location = curs_loc
                 #bpy.ops.transform.rotate('INVOKE_DEFAULT', axis=rot_axis)
-                bpy.ops.transform.rotate('INVOKE_DEFAULT', orient_matrix=o_mat)
+                bpy.ops.transform.rotate('INVOKE_DEFAULT', orient_matrix=o_mat,
+                        orient_axis='Z', constraint_axis=(False, False, True))
 
             #===========================================
             # Check for click on "Add Selected" Button
@@ -1321,7 +1324,7 @@ class XEDIT_OT_free_rotate(bpy.types.Operator):
                                     self.pt_cnt += 1
                                     self.menu.change_menu(self.pt_cnt)
                                     if self.pt_cnt > 1:
-                                        RotDat.axis_lock = None
+                                        TransDat.axis_lock = None
                                         updatelock_pts(self, self.pts)
                                         set_piv(self)
                                         #if self.pt_cnt
@@ -1410,9 +1413,9 @@ class XEDIT_OT_free_rotate(bpy.types.Operator):
             axis_key_check(self, 'Z')
 
             '''
-            elif event.type == 'D' and event.value == 'RELEASE':
-                # open debug console
-                __import__('code').interact(local=dict(globals(), **locals()))
+        elif event.type == 'D' and event.value == 'RELEASE':
+            # open debug console
+            __import__('code').interact(local=dict(globals(), **locals()))
             '''
 
         elif event.type == 'G' and event.value == 'RELEASE':
@@ -1446,8 +1449,8 @@ class XEDIT_OT_free_rotate(bpy.types.Operator):
 
             # Add the region OpenGL drawing callback
             # draw in view space with 'POST_VIEW' and 'PRE_VIEW'
-            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_callback_px,
-                    args, 'WINDOW', 'POST_PIXEL')
+            self._handle = bpy.types.SpaceView3D.draw_handler_add(
+                    draw_callback_px, args, 'WINDOW', 'POST_PIXEL')
 
             self.settings_backup = backup_blender_settings()
             self.mouse_co = Vector((event.mouse_region_x, event.mouse_region_y))
@@ -1481,8 +1484,9 @@ class XEDIT_OT_free_rotate(bpy.types.Operator):
 
             context.window_manager.modal_handler_add(self)
 
-            init_ref_pts(self)
             init_blender_settings()
+            init_ref_pts(self)
+            set_transform_data_none()
             editmode_refresh()
             #print("Add-on started")  # debug
             self.add_rm_btn.set_text("Add Selected")
